/*
 * Copyright 2010 Mihai Niculescu <q.quark@gmail.com>
 *
 * This file is part of EqualX Project (https://launchpad.net/equalx/)
 *
 * EqualX is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EqualX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H


#include <QMap>
#include <QMainWindow>
#include <QProcess>
#include <QTimer>
#include <QTime>

#include "defines.h"

class QAction;
class QActionGroup;
class QButtonGroup;
class QCloseEvent;
class QMenu;
class QModelIndex;
class QMovie;
class QPushButton;
class QToolButton;
class QSignalMapper;
class QProcess;

class DialogPreferences;
class DialogReplace;
class DialogAbout;
class EquationView;
class FrameMathTab;
class LatexEditor;
class WidgetFind;
class WidgetToolMenu;

namespace EqualX {
class RenderEngine;
class FileInfo;
}


namespace Ui
{
class MainWindowClass;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

    void setWindowModified(bool mod=true);

public slots:
    void updateEquation(); // updates tex, png, and scene

    void onFindTextChanged(const QString &exp);

    void showFindDialog();
    void showReplaceDialog();
    void insertSymbol(QAction *action=0);

    void copyEquationToClipboard();
    void pasteEquationFromClipboard();

private slots:
    void equationChanged();

    void newEquation();
    void onAbout();
    void onDeleteSelection(); // delete selected text from editor
    bool onPreferences(); // show preferences dialog
    void open();
    void openRecentFile();
    bool save();
    bool saveAs();

    void loadEquation(const QString &fileName);

    void preferencesAccepted();
    void renderStarted();
    void renderFinished(int exitCode, QProcess::ExitStatus exitStatus);

    void writeSettings(); //read all settings from preferencesDialog and write them to the Settings File
    void readSettings(); // read all settings from the Settings File and update preferencesDialog

protected:
    void showEquation();
    void clearCurrentHighlight();
    void setActions();
    void setSymbolsPanel();
    void setMathPanelTabsActions();
    void setConverters();

    void setSignals();
    void setCurrentFile(const QString &fileName);

    bool wishToSave(); // if user wishes to save modifications

    bool saveEquation(const QString &fileName);

    void closeEvent(QCloseEvent *);


    /*
        Adds ToolButtons with Actions conforming the file found by file name at eqImage
    to the widget
    */
    void addButtonToMathTabs(const QString &eqImage, QWidget *widget);
    void addButtonToMathTabs(const QStringList &list, QWidget *widget);

    void updateUI(); // update Main Window UI according to settings
    void updateRecentFileActions();
    QString strippedName(const QString &fullFileName);

private:
    void checkRequirements(); // check if the requirements are fulfilled

    /*  MainWindow widgets */
    Ui::MainWindowClass *ui;

    EqualX::FileInfo *equation;
    EquationView *equationView;
    DialogReplace *replaceDialog;
    DialogPreferences *preferencesDialog;
    DialogAbout *aboutDialog;
    LatexEditor *editor;
    WidgetFind *findWidget;
    FrameMathTab *templateTab;
    QButtonGroup* renderModeButtonGroup;
    QMovie* movieWorkInProgress;
    QTimer mAutoUpdateTimer;

    QAction *recentFileActions[MAX_RECENT_FILES];
    QSignalMapper *signalMapper;
    EqualX::RenderEngine* renderer;

    /* Symbols Menus */
    WidgetToolMenu *toolDeco;
    WidgetToolMenu *toolFunc;
    WidgetToolMenu *toolDelim;
    WidgetToolMenu *toolAccents;
    WidgetToolMenu *toolOps;
    WidgetToolMenu *toolRelations;
    WidgetToolMenu *toolMisc;
    WidgetToolMenu *toolSets;
    WidgetToolMenu *toolGreek;
    WidgetToolMenu *toolGreekBig;
    WidgetToolMenu *toolSums;
    WidgetToolMenu *toolIntegrals;
    WidgetToolMenu *toolProd;
    WidgetToolMenu *toolArrows;
    WidgetToolMenu *toolSpacing;

    // data
    QString curFile; // current file

    // flags
    bool highlightAll;
};

#endif // MAINWINDOW_H
